# Necessary imports
from scipy.io.matlab.mio import savemat
from pymodelica import compile_fmux
from pymodelica.common.io import ResultDymolaTextual
import matplotlib.pyplot as plt
from pyjmi import CasadiModel

# Choose model
model_name = "STWF"
model_name = "STFE"
model_name = "DTWF"
model_name = "DTFE"
model_name += ".mo"


if model_name == "DTWF.mo":
	initGuess = ResultDymolaTextual('dymola_dt_wf_turn.txt')
	model = compile_fmux('turn_DTWF',['turn.mop','vehiclemodels.mo'])
elif model_name == "STWF.mo": 
	initGuess = ResultDymolaTextual('dymola_st_wf_turn.txt')
	model = compile_fmux('turn_STWF',['turn.mop','vehiclemodels.mo'])
elif model_name == "DTFE.mo": 
	initGuess = ResultDymolaTextual('dymola_dt_fe_turn.txt')
	model = compile_fmux('turn_DTFE',['turn.mop','vehiclemodels.mo'])
elif model_name == "STFE.mo": 
	initGuess = ResultDymolaTextual('dymola_st_fe_turn.txt')
	model = compile_fmux('turn_STFE',['turn.mop','vehiclemodels.mo'])

object = CasadiModel(model)
#Optimization options
opts = object.optimize_options()
opts['IPOPT_options']['linear_solver'] = "ma57"	#Linear solver
opts['IPOPT_options']['max_iter'] = 1000	#Max iterations
opts['IPOPT_options']['tol'] = 1e-10	#Tolerance
opts['n_e'] = 150	#nbr elements
opts['n_cp'] = 3	#nbr collocation points

opts['init_traj'] = initGuess
res = object.optimize(options=opts)

# write solver statistics to file
f = open('solver_stats.txt','w')
f.writelines('return status, number of iterations, final objective value, execution time \n')
f.writelines('(0: opt found, -1: max iter, -2: rest fail) \n')
f.writelines('\nPhase 1:   ')
f.writelines(str(res.solver.get_ipopt_statistics()))
f.close()

if model_name[0:2] == "DT":
	
	# extract variables for DT
	X = res['car.X']
	Y = res['car.Y']
	psi = res['car.psi']
	r = res['car.r']
	vx = res['car.vx']
	vy = res['car.vy']
	phi = res['car.phi']
	dphi = res['car.dphi']
	theta = res['car.theta']
	dtheta = res['car.dtheta']
	ddelta = res['ddelta']
	delta = res['car.delta']
	alpha1 = res['car.alpha1']
	alpha2 = res['car.alpha2']
	alpha3 = res['car.alpha3']
	alpha4 = res['car.alpha4']
	kappa1 = res['car.kappa1']
	kappa2 = res['car.kappa2']
	kappa3 = res['car.kappa3']
	kappa4 = res['car.kappa4']
	omega1 = res['car.omega1']
	omega2 = res['car.omega2']
	omega3 = res['car.omega3']
	omega4 = res['car.omega4']
	Fy1 = res['car.Fy1']
	Fy2 = res['car.Fy2']
	Fy3 = res['car.Fy3']
	Fy4 = res['car.Fy4']
	Fx1 = res['car.Fx1']
	Fx2 = res['car.Fx2']
	Fx3 = res['car.Fx3']
	Fx4 = res['car.Fx4']
	Fz1 = res['car.Fz1']
	Fz2 = res['car.Fz2']
	Fz3 = res['car.Fz3']
	Fz4 = res['car.Fz4']
	Twf = res['car.Twf']
	Twr = res['car.Twr']
	FX = res['car.Fx_g']
	FY = res['car.Fy_g']
	MZ = res['car.Mz_g']
	tf = res['finalTime']
	t = res['time']
	#print("tf = %s" % tf)
	
	# save res to mat-file
	savemat('turn_result.mat',{'phi':phi,'dphi':dphi,'theta':theta,'dtheta':theta,'X':X,'Y':Y,'psi':psi,'r':r,'vx':vx,'vy':vy,'delta':delta,'ddelta':ddelta,'alpha1':alpha1,'alpha2':alpha2,'alpha3':alpha3,'alpha4':alpha4,'kappa1':kappa1,'kappa2':kappa2,'kappa3':kappa3,'kappa4':kappa4,'omega1':omega1,'omega2':omega2,'omega3':omega3,'omega4':omega4,'Fy1':Fy1,'Fy2':Fy2,'Fy3':Fy3,'Fy4':Fy4,'Fx1':Fx1,'Fx2':Fx2,'Fx3':Fx3,'Fx4':Fx4,'Fz1':Fz1,'Fz2':Fz2,'Fz3':Fz3,'Fz4':Fz4,'Twf':Twf,'Twr':Twr,'FX':FX,'FY':FY,'MZ':MZ,'t':t},appendmat=False)
	
	
else:
	# extract variables for ST
	X = res['car.X']
	Y = res['car.Y']
	psi = res['car.psi']
	r = res['car.r']
	vx = res['car.vx']
	vy = res['car.vy']
	ddelta = res['ddelta']
	delta = res['car.delta']
	alphaf = res['car.alphaf']
	alphar = res['car.alphar']
	kappaf = res['car.kappaf']
	kappar = res['car.kappar']
	omegaf = res['car.omegaf']
	omegar = res['car.omegar']
	Fyf = res['car.Fyf']
	Fyr = res['car.Fyr']
	Fxf = res['car.Fxf']
	Fxr = res['car.Fxr']
	Twf = res['car.Twf']
	Twr = res['car.Twr']
	FX = res['car.Fx_g']
	FY = res['car.Fy_g']
	MZ = res['car.Mz_g']
	tf = res['finalTime']
	t = res['time']
	#print("tf = %s" % tf)
	
	# save res to mat-file
	savemat('turn_result.mat',{'X':X,'Y':Y,'psi':psi,'r':r,'vx':vx,'vy':vy,'delta':delta,'ddelta':ddelta,'alphaf':alphaf,'alphar':alphar,'kappaf':kappaf,'kappar':kappar,'omegaf':omegaf,'omegar':omegar,'Fyf':Fyf,'Fyr':Fyr,'Fxf':Fxf,'Fxr':Fxr,'Twf':Twf,'Twr':Twr,'FX':FX,'FY':FY,'MZ':MZ,'t':t},appendmat=False)
	
	
	
