

model DTFE
//////// VEHICLE MODEL ////////

  constant Real pi = Modelica.Constants.pi;

  // init parameters (adjust init parameters above instead)
  parameter Real X0 = 0;
  parameter Real Y0 = 0;
  parameter Real vx0 = 25;
  parameter Real psi0 = 0;
  // vehicle properties
  parameter Real l = 2.8;
  parameter Real lf = 1.3;
  parameter Real lr = l-lf;
  parameter Real w = 1.6/2;
  parameter Real m = 2100;
  parameter Real Izz = 3900;
  parameter Real g = 9.82;
  parameter Real Fzf0 = m*g*lr/l;
  parameter Real Fzr0 = m*g*lf/l;
  parameter Real Fz10 = Fzf0/2;
  parameter Real Fz20 = Fzf0/2;
  parameter Real Fz30 = Fzr0/2;
  parameter Real Fz40 = Fzr0/2;

  parameter Real Ixx = 765;
  parameter Real h = 0.5;
  parameter Real Iyy = 3477;
  parameter Real Dphi = 16000;
  parameter Real Kphi = 178000;

  parameter Real Kth = 363540;
  parameter Real Dth = 30960; 
 

  // tire parameters
  parameter Real Rw = .3; // wheel radius
  parameter Real Iw = 2;
  parameter Real sig = .3;

  // Dry Asphalt
  // Front tire model parameters
  parameter Real muxf = 1.1959;
  parameter Real Bxf = 11.6848;
  parameter Real Cxf = 1.685;
  parameter Real Exf = 0.37729;
 
  parameter Real muyf = 0.93476;
  parameter Real Byf = 8.8626;
  parameter Real Cyf = 1.193;
  parameter Real Eyf = -1.2076;
 
  parameter Real Cxaf = 1.092;
  parameter Real Bx1f = 12.35;
  parameter Real Bx2f = -10.77;
 
  parameter Real Cykf = 1.081;
  parameter Real By1f = 6.461;
  parameter Real By2f = 4.196;
 
  // Rear tire model parameter
  parameter Real muxr = 1.2027;
  parameter Real Bxr = 11.1217;
  parameter Real Cxr = 1.685;
  parameter Real Exr = 0.36192;
 
  parameter Real muyr = 0.96146;
  parameter Real Byr = 9.3016;
  parameter Real Cyr = 1.193;
  parameter Real Eyr = -1.1087;
 
  parameter Real Cxar = 1.092;
  parameter Real Bx1r = 12.35;
  parameter Real Bx2r = -10.77;
 
  parameter Real Cykr = 1.081;
  parameter Real By1r = 6.461;
  parameter Real By2r = 4.196;

  parameter Real Twfmin = -muxf*m*g*Rw;
  parameter Real Twrmin = -muxr*m*g*Rw;
  parameter Real Twfmax = 0;
  parameter Real Twrmax = muxr*Fzr0*Rw;
  parameter Real Fxfmax = muxf*m*g;
  parameter Real Fxrmax = muxr*m*g;
  parameter Real Fyfmax = muyf*m*g;
  parameter Real Fyrmax = muyr*m*g;

  // inputs
  Modelica.Blocks.Interfaces.RealInput delta(min=-.5, max=.5, nominal=.2);
  Modelica.Blocks.Interfaces.RealInput Twf(min=Twfmin, max=Twfmax, nominal=Twrmax);
  Modelica.Blocks.Interfaces.RealInput Twr(min=Twrmin, max=Twrmax, nominal=Twrmax);

  // variables
  Real X(nominal=50, start=X0,fixed=true);
  Real Y(nominal=10, start=Y0,fixed=true);
  Real psi(nominal=pi, start=psi0,fixed=true);
  Real vy(nominal=1, start=0, fixed=true);
  Real vx(nominal=10, start=vx0,fixed=true, min=0.1, max=100);
  Real r(nominal=1, start=0, fixed=true);
  Real phi(start=0,fixed=true);
  Real dphi(start=0,fixed=true);
  Real theta(start=0,fixed=true);
  Real dtheta(start=0,fixed=true);

  Real vx1(nominal=10, start=vx0);
  Real vx2(nominal=10, start=vx0);
  Real vx3(nominal=10, start=vx0);
  Real vx4(nominal=10, start=vx0);

  Real vy1(nominal=1, start=0);
  Real vy2(nominal=1, start=0);
  Real vy3(nominal=1, start=0);
  Real vy4(nominal=1, start=0);

  Real alpha1(nominal=.2, start=0, fixed=true);
  Real alpha2(nominal=.2, start=0, fixed=true);
  Real alpha3(nominal=.2, start=0, fixed=true);
  Real alpha4(nominal=.2, start=0, fixed=true);


  Real omega1(nominal=30, start=vx0/Rw,fixed=true, min=0);
  Real omega2(nominal=30, start=vx0/Rw,fixed=true, min=0);
  Real omega3(nominal=30, start=vx0/Rw,fixed=true, min=0);
  Real omega4(nominal=30, start=vx0/Rw,fixed=true, min=0);

  Real kappa1(nominal=.3, start=0);
  Real kappa2(nominal=.3, start=0);
  Real kappa3(nominal=.3, start=0);
  Real kappa4(nominal=.3, start=0);

  Real Fy10(nominal=Fyfmax, start=0);
  Real Fy20(nominal=Fyfmax, start=0);
  Real Fy30(nominal=Fyrmax, start=0);
  Real Fy40(nominal=Fyrmax, start=0);

  Real Fy1(nominal=Fyfmax, start=0, min=-Fyfmax, max=Fyfmax);
  Real Fy2(nominal=Fyfmax, start=0, min=-Fyfmax, max=Fyfmax);
  Real Fy3(nominal=Fyrmax, start=0, min=-Fyrmax, max=Fyrmax);
  Real Fy4(nominal=Fyrmax, start=0, min=-Fyrmax, max=Fyrmax);

  Real Fx1(nominal=Fxfmax, start=0, min=-Fxfmax, max=Fxfmax);
  Real Fx2(nominal=Fxfmax, start=0, min=-Fxfmax, max=Fxfmax);
  Real Fx3(nominal=Fxrmax, start=0, min=-Fxrmax, max=Fxrmax);
  Real Fx4(nominal=Fxrmax, start=0, min=-Fxrmax, max=Fxrmax);

  Real Fz1(nominal=Fz10, start=Fz10, min=0, max=m*g);
  Real Fz2(nominal=Fz20, start=Fz20, min=0, max=m*g);
  Real Fz3(nominal=Fz30, start=Fz30, min=0, max=m*g);
  Real Fz4(nominal=Fz40, start=Fz40, min=0, max=m*g);

  Real Mz_g(start=0,nominal=Fxfmax);
  Real Fx_g(start=0,nominal=Fxfmax);
  Real Fy_g(start=0,nominal=Fyfmax);

  Real Tw1(min=Twfmin, max=Twfmax, nominal=Twrmax);
  Real Tw2(min=Twfmin, max=Twfmax, nominal=Twrmax);
  Real Tw3(min=Twrmin, max=Twrmax, nominal=Twrmax);
  Real Tw4(min=Twrmin, max=Twrmax, nominal=Twrmax);
  
equation

  // motion equations
  Fx_g = (Fx1+Fx2)*cos(delta) - (Fy1+Fy2)*sin(delta)  + Fx3 + Fx4;
  Fy_g = (Fx1+Fx2)*sin(delta) + (Fy1+Fy2)*cos(delta)  + Fy3 + Fy4;
  Mz_g = lf*( (Fx1+Fx2)*sin(delta) + (Fy1+Fy2)*cos(delta) ) - lr*(Fy3+Fy4) + w*( (-Fx1+Fx2)*cos(delta) + (Fy1-Fy2)*sin(delta) - Fx3 + Fx4);

  ( -vy*r + der(vx) + sin(phi)*der(r)*h - sin(theta)*cos(phi)*h*(r^2 + dphi^2 + dtheta^2) + 2*cos(phi)*h*dphi*r + cos(theta)*cos(phi)*h*der(dtheta) - 2*cos(theta)*sin(phi)*dtheta*dphi*h - sin(theta)*sin(phi)*h*der(dphi))*m = Fx_g;
  
  ( der(vy) + vx*r + sin(theta)*cos(phi)*h*der(r) + sin(phi)*h*r^2 + 2*(cos(theta)*cos(phi)*h*dtheta*r - sin(theta)*sin(phi)*h*dphi*r) + sin(phi)*h*dphi^2 - cos(phi)*h*der(dphi))*m = Fy_g;
  
  der(r)*( sin(theta)^2*Ixx + cos(theta)^2*( Izz*cos(phi)^2 + Iyy*sin(phi)^2 ) ) = Mz_g - Fx_g*h*sin(phi) - sin(theta)*cos(phi)*h*Fy_g;
  
  der(dphi)*( Ixx*cos(theta)^2 + sin(theta)^2*sin(phi)^2*Iyy + sin(theta)^2*cos(phi)^2*Izz ) + Kphi*phi + Dphi*dphi = Fy_g*h*cos(phi)*cos(theta) + m*g*h*sin(phi) + r*( r*(Iyy - Izz)*sin(phi)*cos(phi)*cos(theta) + dphi*sin(theta)*sin(phi)*cos(phi)*(Iyy-Izz) + dtheta*(cos(phi)^2*Iyy+sin(phi)^2*Izz) );
  
  der(dtheta)*( Iyy*cos(phi)^2 + Izz*sin(phi)^2 ) + Dth*dtheta + Kth*theta = m*g*h*sin(theta)*cos(phi) - cos(theta)*cos(phi)*h*Fx_g + r*( r*sin(theta)*cos(theta)*( Ixx - Iyy + cos(phi)^2*(Iyy - Izz) ) - dphi*(cos(theta)^2*Ixx + sin(phi)^2*sin(theta)^2*Iyy + sin(theta)^2*cos(phi)^2*Izz) - dtheta*(sin(theta)*sin(phi)*cos(phi)*(Iyy - Izz)) );

  dphi = der(phi);
  der(theta) = dtheta;

  -w*(Fz1-Fz2) = Kphi/2*phi + Dphi/2*dphi;
  -w*(Fz3-Fz4) = Kphi/2*phi + Dphi/2*dphi;
  (Fz1+Fz2)*lf - (Fz3+Fz4)*lr = Kth*theta + Dth*dtheta;
  Fz1 + Fz2 + Fz3 + Fz4 = m*g;

  Tw1 = Twf/2;
  Tw2 = Twf/2;
  Tw3 = Twr/2;
  Tw4 = Twr/2;

  Tw1 - Iw*der(omega1) - Fx1*Rw = 0;
  Tw2 - Iw*der(omega2) - Fx2*Rw = 0;
  Tw3 - Iw*der(omega3) - Fx3*Rw = 0;
  Tw4 - Iw*der(omega4) - Fx4*Rw = 0;

  // slip angles
  vx1 = (vx-r*w)*cos(delta) + (vy+r*lf)*sin(delta);
  vx2 = (vx+r*w)*cos(delta) + (vy+r*lf)*sin(delta);
  vx3 = vx - r*w;
  vx4 = vx + r*w;

  vy1 = (vy+r*lf)*cos(delta) - (vx-r*w)*sin(delta);
  vy2 = (vy+r*lf)*cos(delta) - (vx+r*w)*sin(delta);
  vy3 = vy - r*lr;
  vy4 = vy - r*lr;

  der(alpha1)*sig/vx1 + alpha1 = -atan(vy1/vx1);
  der(alpha2)*sig/vx2 + alpha2 = -atan(vy2/vx2);
  der(alpha3)*sig/vx3 + alpha3 = -atan(vy3/vx3);
  der(alpha4)*sig/vx4 + alpha4 = -atan(vy4/vx4);

  kappa1 = (Rw*omega1 - vx1)/vx1;
  kappa2 = (Rw*omega2 - vx2)/vx2;
  kappa3 = (Rw*omega3 - vx3)/vx3;
  kappa4 = (Rw*omega4 - vx4)/vx4;

  // tire forces
  Fx1 = muxf*Fz1*sin(Cxf*atan(Bxf*kappa1 - Exf*(Bxf*kappa1 - atan(Bxf*kappa1))));
  Fx2 = muxf*Fz2*sin(Cxf*atan(Bxf*kappa2 - Exf*(Bxf*kappa2 - atan(Bxf*kappa2))));
  Fx3 = muxr*Fz3*sin(Cxr*atan(Bxr*kappa3 - Exr*(Bxr*kappa3 - atan(Bxr*kappa3))));
  Fx4 = muxr*Fz4*sin(Cxr*atan(Bxr*kappa4 - Exr*(Bxr*kappa4 - atan(Bxr*kappa4))));
  
  Fy10 = muyf*Fz1*sin(Cyf*atan(Byf*alpha1 - Eyf*(Byf*alpha1 - atan(Byf*alpha1))));
  Fy20 = muyf*Fz2*sin(Cyf*atan(Byf*alpha2 - Eyf*(Byf*alpha2 - atan(Byf*alpha2))));
  Fy30 = muyr*Fz3*sin(Cyr*atan(Byr*alpha3 - Eyr*(Byr*alpha3 - atan(Byr*alpha3))));
  Fy40 = muyr*Fz4*sin(Cyr*atan(Byr*alpha4 - Eyr*(Byr*alpha4 - atan(Byr*alpha4))));

  Fy1 = Fy10*sqrt(1 - .9999*(sin(Cxf*atan(Bxf*kappa1 - Exf*(Bxf*kappa1 - atan(Bxf*kappa1)))))^2);
  Fy2 = Fy20*sqrt(1 - .9999*(sin(Cxf*atan(Bxf*kappa2 - Exf*(Bxf*kappa2 - atan(Bxf*kappa2)))))^2);
  Fy3 = Fy30*sqrt(1 - .9999*(sin(Cxr*atan(Bxr*kappa3 - Exr*(Bxr*kappa3 - atan(Bxr*kappa3)))))^2);
  Fy4 = Fy40*sqrt(1 - .9999*(sin(Cxr*atan(Bxr*kappa4 - Exr*(Bxr*kappa4 - atan(Bxr*kappa4)))))^2);

  // global coordinates
  der(psi) = r;
  der(X) = vx*cos(psi) - vy*sin(psi);
  der(Y) = vx*sin(psi) + vy*cos(psi);

end DTFE;


model STFE
//////// VEHICLE MODEL ////////

  constant Real pi = Modelica.Constants.pi;

  // init parameters (adjust init parameters above instead)
  parameter Real X0 = 0;
  parameter Real Y0 = 0;
  parameter Real vx0 = 25;
  parameter Real psi0 = 0;
  // vehicle properties
  parameter Real l = 2.8;
  parameter Real lf = 1.3;
  parameter Real lr = l-lf;
  parameter Real m = 2100;
  parameter Real Iz = 3900;
  parameter Real g = 9.82;
  parameter Real Fzf = m*g*lr/l;
  parameter Real Fzr = m*g*lf/l;

  // tire parameters
  parameter Real Rw = .3; // wheel radius
  parameter Real Iw = 2;
  parameter Real sig = .3;

  // Dry Asphalt
  // Front tire model parameters
  parameter Real muxf = 1.1959;
  parameter Real Bxf = 11.6848;
  parameter Real Cxf = 1.685;
  parameter Real Exf = 0.37729;
 
  parameter Real muyf = 0.93476;
  parameter Real Byf = 8.8626;
  parameter Real Cyf = 1.193;
  parameter Real Eyf = -1.2076;
 
  parameter Real Cxaf = 1.092;
  parameter Real Bx1f = 12.35;
  parameter Real Bx2f = -10.77;
 
  parameter Real Cykf = 1.081;
  parameter Real By1f = 6.461;
  parameter Real By2f = 4.196;
 
  // Rear tire model parameter
  parameter Real muxr = 1.2027;
  parameter Real Bxr = 11.1217;
  parameter Real Cxr = 1.685;
  parameter Real Exr = 0.36192;
 
  parameter Real muyr = 0.96146;
  parameter Real Byr = 9.3016;
  parameter Real Cyr = 1.193;
  parameter Real Eyr = -1.1087;
 
  parameter Real Cxar = 1.092;
  parameter Real Bx1r = 12.35;
  parameter Real Bx2r = -10.77;
 
  parameter Real Cykr = 1.081;
  parameter Real By1r = 6.461;
  parameter Real By2r = 4.196;

  parameter Real Twfmin = -muxf*m*g*Rw;
  parameter Real Twrmin = -muxr*m*g*Rw;
  parameter Real Twfmax = 0;
  parameter Real Twrmax = muxr*Fzr*Rw;
  parameter Real Fxfmax = muxf*Fzf;
  parameter Real Fxrmax = muxr*Fzr;
  parameter Real Fyfmax = muyf*Fzf;
  parameter Real Fyrmax = muyr*Fzr;

  // inputs
  Modelica.Blocks.Interfaces.RealInput delta(min=-.5, max=.5, nominal=.2);
  Modelica.Blocks.Interfaces.RealInput Twf(min=Twfmin, max=Twfmax, nominal=Twrmax);
  Modelica.Blocks.Interfaces.RealInput Twr(min=Twrmin, max=Twrmax, nominal=Twrmax);

  // variables
  Real X(nominal=50, start=X0,fixed=true);
  Real Y(nominal=10, start=Y0,fixed=true);
  Real psi(nominal=pi, start=psi0,fixed=true);
  Real vy(nominal=1, start=0, fixed=true);
  Real vx(nominal=10, start=vx0,fixed=true, min=0.1, max=100);
  Real r(nominal=1, start=0, fixed=true);
  Real alphaf(nominal=.2, start=0, fixed=true);
  Real alphar(nominal=.2, start=0, fixed=true);
  Real Fyf0(nominal=Fyfmax, start=0);
  Real Fyr0(nominal=Fyrmax, start=0);
  Real Fyf(nominal=Fyfmax, start=0, min=-Fyfmax, max=Fyfmax);
  Real Fyr(nominal=Fyrmax, start=0, min=-Fyrmax, max=Fyrmax);
  Real Fxf(nominal=Fxfmax, start=0, min=-Fxfmax, max=Fxfmax);
  Real Fxr(nominal=Fxrmax, start=0, min=-Fxrmax, max=Fxrmax);
  Real omegaf(nominal=30, start=vx0/Rw,fixed=true, min=0);
  Real omegar(nominal=30, start=vx0/Rw,fixed=true, min=0);
  Real kappaf(nominal=.3, start=0);
  Real kappar(nominal=.3, start=0);
  Real vxf(nominal=10, start=vx0);
  Real vxr(nominal=10, start=vx0);
  Real Mz_g(start=0,nominal=Fxfmax);
  Real Fx_g(start=0,nominal=Fxfmax);
  Real Fy_g(start=0,nominal=Fyfmax);
  
equation
  Mz_g = (lf*Fyf*cos(delta) - lr*Fyr + lf*Fxf*sin(delta));
  Fx_g = (Fxf*cos(delta) + Fxr - Fyf*sin(delta));
  Fy_g = (Fyf*cos(delta) + Fyr + Fxf*sin(delta));

  // motion equations
  (-vy*r+der(vx))*m = Fx_g;
  (der(vy)+vx*r)*m = Fy_g;
  der(r)*Iz = Mz_g; 

  Twf - 2*Iw*der(omegaf) - Fxf*Rw = 0;
  Twr - 2*Iw*der(omegar) - Fxr*Rw = 0;

  // slip angles
  der(alphaf)*sig/vxf + alphaf = delta - atan((vy + lf*r)/vx);
  der(alphar)*sig/vxr + alphar = -atan((vy - lr*r)/vx);

  vxf = vx*cos(delta) + (vy+lf*r)*sin(delta);
  vxr = vx;

  kappaf = (Rw*omegaf - vxf)/vxf;
  kappar = (Rw*omegar - vxr)/vxr;

  // tire forces
  Fxf = muxf*Fzf*sin(Cxf*atan(Bxf*kappaf - Exf*(Bxf*kappaf - atan(Bxf*kappaf))));
  Fxr = muxr*Fzr*sin(Cxr*atan(Bxr*kappar - Exr*(Bxr*kappar - atan(Bxr*kappar))));
  
  Fyf0 = muyf*Fzf*sin(Cyf*atan(Byf*alphaf - Eyf*(Byf*alphaf - atan(Byf*alphaf))));
  Fyr0 = muyr*Fzr*sin(Cyr*atan(Byr*alphar - Eyr*(Byr*alphar - atan(Byr*alphar))));

  Fyf = Fyf0*sqrt(1 - .9999*(Fxf/Fxfmax)^2);
  Fyr = Fyr0*sqrt(1 - .9999*(Fxr/Fxrmax)^2);

  // global coordinates
  der(psi) = r;
  der(X) = vx*cos(psi) - vy*sin(psi);
  der(Y) = vx*sin(psi) + vy*cos(psi);

end STFE;


model STWF

//////// STWF MODEL ////////



  constant Real pi = Modelica.Constants.pi;

  // init parameters (adjust init parameters above instead)
  parameter Real X0 = 0;
  parameter Real Y0 = 0;
  parameter Real vx0 = 25;
  parameter Real psi0 = 0;
  // vehicle properties
  parameter Real l = 2.8;
  parameter Real lf = 1.3;
  parameter Real lr = l-lf;
  parameter Real m = 2100;
  parameter Real Iz = 3900;
  parameter Real g = 9.82;
  parameter Real Fzf = m*g*lr/l;
  parameter Real Fzr = m*g*lf/l;

  // tire parameters
  parameter Real Rw = .3; // wheel radius
  parameter Real Iw = 2;
  parameter Real sig = .3;

  // Dry Asphalt
  // Front tire model parameters
  parameter Real muxf = 1.1959;
  parameter Real Bxf = 11.6848;
  parameter Real Cxf = 1.685;
  parameter Real Exf = 0.37729;
 
  parameter Real muyf = 0.93476;
  parameter Real Byf = 8.8626;
  parameter Real Cyf = 1.193;
  parameter Real Eyf = -1.2076;
 
  parameter Real Cxaf = 1.092;
  parameter Real Bx1f = 12.35;
  parameter Real Bx2f = -10.77;
 
  parameter Real Cykf = 1.081;
  parameter Real By1f = 6.461;
  parameter Real By2f = 4.196;
 
  // Rear tire model parameter
  parameter Real muxr = 1.2027;
  parameter Real Bxr = 11.1217;
  parameter Real Cxr = 1.685;
  parameter Real Exr = 0.36192;
 
  parameter Real muyr = 0.96146;
  parameter Real Byr = 9.3016;
  parameter Real Cyr = 1.193;
  parameter Real Eyr = -1.1087;
 
  parameter Real Cxar = 1.092;
  parameter Real Bx1r = 12.35;
  parameter Real Bx2r = -10.77;
 
  parameter Real Cykr = 1.081;
  parameter Real By1r = 6.461;
  parameter Real By2r = 4.196;

  parameter Real Twfmin = -muxf*m*g*Rw;
  parameter Real Twrmin = -muxr*m*g*Rw;
  parameter Real Twfmax = 0;
  parameter Real Twrmax = muxr*Fzr*Rw;
  parameter Real Fxfmax = muxf*Fzf;
  parameter Real Fxrmax = muxr*Fzr;
  parameter Real Fyfmax = muyf*Fzf;
  parameter Real Fyrmax = muyr*Fzr;

  // inputs
  Modelica.Blocks.Interfaces.RealInput delta(min=-.5, max=.5, nominal=.2);
  Modelica.Blocks.Interfaces.RealInput Twf(min=Twfmin, max=Twfmax, nominal=Twrmax);
  Modelica.Blocks.Interfaces.RealInput Twr(min=Twrmin, max=Twrmax, nominal=Twrmax);

  // variables
  Real X(nominal=50, start=X0,fixed=true);
  Real Y(nominal=10, start=Y0,fixed=true);
  Real psi(nominal=pi, start=psi0,fixed=true);
  Real vy(nominal=1, start=0, fixed=true);
  Real vx(nominal=10, start=vx0,fixed=true, min=0.1, max=100);
  Real r(nominal=1, start=0, fixed=true);
  Real alphaf(nominal=.2, start=0, fixed=true);
  Real alphar(nominal=.2, start=0, fixed=true);
  Real Fxf0(nominal=Fxfmax, start=0);
  Real Fxr0(nominal=Fxrmax, start=0);
  Real Fyf0(nominal=Fyfmax, start=0);
  Real Fyr0(nominal=Fyrmax, start=0);
  Real Fyf(nominal=Fyfmax, start=0, min=-Fyfmax, max=Fyfmax);
  Real Fyr(nominal=Fyrmax, start=0, min=-Fyrmax, max=Fyrmax);
  Real Fxf(nominal=Fxfmax, start=0, min=-Fxfmax, max=Fxfmax);
  Real Fxr(nominal=Fxrmax, start=0, min=-Fxrmax, max=Fxrmax);
  Real omegaf(nominal=30, start=vx0/Rw,fixed=true, min=0);
  Real omegar(nominal=30, start=vx0/Rw,fixed=true, min=0);
  Real kappaf(nominal=.3, start=0);
  Real kappar(nominal=.3, start=0);
  Real Gxaf(nominal=1, start=1);
  Real Gxar(nominal=1, start=1);
  Real Gykf(nominal=1, start=1);
  Real Gykr(nominal=1, start=1);
  Real Bxaf(nominal=Bx1f);
  Real Bxar(nominal=Bx1r);
  Real Bykf(nominal=By1f);
  Real Bykr(nominal=By1r);
  Real vxf(nominal=10, start=vx0);
  Real vxr(nominal=10, start=vx0);
  Real Mz_g(start=0,nominal=Fxfmax);
  Real Fx_g(start=0,nominal=Fxfmax);
  Real Fy_g(start=0,nominal=Fyfmax);
  
equation
  Mz_g = (lf*Fyf*cos(delta) - lr*Fyr + lf*Fxf*sin(delta));
  Fx_g = (Fxf*cos(delta) + Fxr - Fyf*sin(delta));
  Fy_g = (Fyf*cos(delta) + Fyr + Fxf*sin(delta));

  // motion equations
  (-vy*r+der(vx))*m = Fx_g;
  (der(vy)+vx*r)*m = Fy_g;
  der(r)*Iz = Mz_g; 

  Twf - 2*Iw*der(omegaf) - Fxf*Rw = 0;
  Twr - 2*Iw*der(omegar) - Fxr*Rw = 0;

  // slip angles
  der(alphaf)*sig/vxf + alphaf = delta - atan((vy + lf*r)/vx);
  der(alphar)*sig/vxr + alphar = -atan((vy - lr*r)/vx);

  vxf = vx*cos(delta) + (vy+lf*r)*sin(delta);
  vxr = vx;

  kappaf = (Rw*omegaf - vxf)/vxf;
  kappar = (Rw*omegar - vxr)/vxr;

  // tire forces
  Bxaf = Bx1f*cos(atan(Bx2f*kappaf));
  Bxar = Bx1r*cos(atan(Bx2r*kappar));

  Gxaf = cos(Cxaf*atan(Bxaf*alphaf));
  Gxar = cos(Cxar*atan(Bxar*alphar));

  Bykf = By1f*cos(atan(By2f*(alphaf)));
  Bykr = By1r*cos(atan(By2r*(alphar)));

  Gykf = cos(Cykf*atan(Bykf*kappaf));
  Gykr = cos(Cykr*atan(Bykr*kappar));

  Fxf0 = muxf*Fzf*sin(Cxf*atan(Bxf*kappaf - Exf*(Bxf*kappaf - atan(Bxf*kappaf))));
  Fxr0 = muxr*Fzr*sin(Cxr*atan(Bxr*kappar - Exr*(Bxr*kappar - atan(Bxr*kappar))));
  
  Fyf0 = muyf*Fzf*sin(Cyf*atan(Byf*alphaf - Eyf*(Byf*alphaf - atan(Byf*alphaf))));
  Fyr0 = muyr*Fzr*sin(Cyr*atan(Byr*alphar - Eyr*(Byr*alphar - atan(Byr*alphar))));

  Fxf = Gxaf*Fxf0;
  Fxr = Gxar*Fxr0;

  Fyf = Gykf*Fyf0;
  Fyr = Gykr*Fyr0;

  // global coordinates
  der(psi) = r;
  der(X) = vx*cos(psi) - vy*sin(psi);
  der(Y) = vx*sin(psi) + vy*cos(psi);

end STWF;



model DTWF
//////// VEHICLE MODEL ////////
  constant Real pi = Modelica.Constants.pi;

  // init parameters (adjust init parameters above instead)
  parameter Real X0 = 0;
  parameter Real Y0 = 0;
  parameter Real vx0 = 25;
  parameter Real psi0 = 0;
  // vehicle properties
  parameter Real l = 2.8;
  parameter Real lf = 1.3;
  parameter Real lr = l-lf;
  parameter Real w = 1.6/2;
  parameter Real m = 2100;
  parameter Real Izz = 3900;
  parameter Real g = 9.82;
  parameter Real Fzf0 = m*g*lr/l;
  parameter Real Fzr0 = m*g*lf/l;
  parameter Real Fz10 = Fzf0/2;
  parameter Real Fz20 = Fzf0/2;
  parameter Real Fz30 = Fzr0/2;
  parameter Real Fz40 = Fzr0/2;

  parameter Real Ixx = 765;
  parameter Real h = 0.5;
  parameter Real Iyy = 3477;
  parameter Real Dphi = 16000;
  parameter Real Kphi = 178000;

  parameter Real Kth = 363540;
  parameter Real Dth = 30960; 
  //parameter Real Kth = 363540;
  //parameter Real Dth = 30960; 

  // tire parameters
  parameter Real Rw = .3; // wheel radius
  parameter Real Iw = 2;
  parameter Real sig = .3;

  // Dry Asphalt
  // Front tire model parameters
  parameter Real muxf = 1.1959;
  parameter Real Bxf = 11.6848;
  parameter Real Cxf = 1.685;
  parameter Real Exf = 0.37729;
 
  parameter Real muyf = 0.93476;
  parameter Real Byf = 8.8626;
  parameter Real Cyf = 1.193;
  parameter Real Eyf = -1.2076;
 
  parameter Real Cxaf = 1.092;
  parameter Real Bx1f = 12.35;
  parameter Real Bx2f = -10.77;
 
  parameter Real Cykf = 1.081;
  parameter Real By1f = 6.461;
  parameter Real By2f = 4.196;
 
  // Rear tire model parameter
  parameter Real muxr = 1.2027;
  parameter Real Bxr = 11.1217;
  parameter Real Cxr = 1.685;
  parameter Real Exr = 0.36192;
 
  parameter Real muyr = 0.96146;
  parameter Real Byr = 9.3016;
  parameter Real Cyr = 1.193;
  parameter Real Eyr = -1.1087;
 
  parameter Real Cxar = 1.092;
  parameter Real Bx1r = 12.35;
  parameter Real Bx2r = -10.77;
 
  parameter Real Cykr = 1.081;
  parameter Real By1r = 6.461;
  parameter Real By2r = 4.196;

  parameter Real Twfmin = -muxf*m*g*Rw;
  parameter Real Twrmin = -muxr*m*g*Rw;
  parameter Real Twfmax = 0;
  parameter Real Twrmax = muxr*Fzr0*Rw;
  parameter Real Fxfmax = muxf*m*g;
  parameter Real Fxrmax = muxr*m*g;
  parameter Real Fyfmax = muyf*m*g;
  parameter Real Fyrmax = muyr*m*g;

  // inputs
  Modelica.Blocks.Interfaces.RealInput delta(min=-.5, max=.5, nominal=.2);
  Modelica.Blocks.Interfaces.RealInput Twf(min=Twfmin, max=Twfmax, nominal=Twrmax);
  Modelica.Blocks.Interfaces.RealInput Twr(min=Twrmin, max=Twrmax, nominal=Twrmax);

  // variables
  Real X(nominal=50, start=X0,fixed=true);
  Real Y(nominal=10, start=Y0,fixed=true);
  Real psi(nominal=pi, start=psi0,fixed=true);
  Real vy(nominal=1, start=0, fixed=true);
  Real vx(nominal=10, start=vx0,fixed=true, min=0.1, max=100);
  Real r(nominal=1, start=0, fixed=true);
  Real phi(start=0,fixed=true);
  Real dphi(start=0,fixed=true);
  Real theta(start=0,fixed=true);
  Real dtheta(start=0,fixed=true);

  Real vx1(nominal=10, start=vx0);
  Real vx2(nominal=10, start=vx0);
  Real vx3(nominal=10, start=vx0);
  Real vx4(nominal=10, start=vx0);

  Real vy1(nominal=1, start=0);
  Real vy2(nominal=1, start=0);
  Real vy3(nominal=1, start=0);
  Real vy4(nominal=1, start=0);

  Real alpha1(nominal=.2, start=0, fixed=true);
  Real alpha2(nominal=.2, start=0, fixed=true);
  Real alpha3(nominal=.2, start=0, fixed=true);
  Real alpha4(nominal=.2, start=0, fixed=true);


  Real omega1(nominal=30, start=vx0/Rw,fixed=true, min=0);
  Real omega2(nominal=30, start=vx0/Rw,fixed=true, min=0);
  Real omega3(nominal=30, start=vx0/Rw,fixed=true, min=0);
  Real omega4(nominal=30, start=vx0/Rw,fixed=true, min=0);

  Real kappa1(nominal=.3, start=0);
  Real kappa2(nominal=.3, start=0);
  Real kappa3(nominal=.3, start=0);
  Real kappa4(nominal=.3, start=0);

  Real Fx10(nominal=Fxfmax, start=0);
  Real Fx20(nominal=Fxfmax, start=0);
  Real Fx30(nominal=Fxrmax, start=0);
  Real Fx40(nominal=Fxrmax, start=0);

  Real Fy10(nominal=Fyfmax, start=0);
  Real Fy20(nominal=Fyfmax, start=0);
  Real Fy30(nominal=Fyrmax, start=0);
  Real Fy40(nominal=Fyrmax, start=0);

  Real Fy1(nominal=Fyfmax, start=0, min=-Fyfmax, max=Fyfmax);
  Real Fy2(nominal=Fyfmax, start=0, min=-Fyfmax, max=Fyfmax);
  Real Fy3(nominal=Fyrmax, start=0, min=-Fyrmax, max=Fyrmax);
  Real Fy4(nominal=Fyrmax, start=0, min=-Fyrmax, max=Fyrmax);

  Real Fx1(nominal=Fxfmax, start=0, min=-Fxfmax, max=Fxfmax);
  Real Fx2(nominal=Fxfmax, start=0, min=-Fxfmax, max=Fxfmax);
  Real Fx3(nominal=Fxrmax, start=0, min=-Fxrmax, max=Fxrmax);
  Real Fx4(nominal=Fxrmax, start=0, min=-Fxrmax, max=Fxrmax);

  Real Fz1(nominal=Fz10, start=Fz10, min=0, max=m*g);
  Real Fz2(nominal=Fz20, start=Fz20, min=0, max=m*g);
  Real Fz3(nominal=Fz30, start=Fz30, min=0, max=m*g);
  Real Fz4(nominal=Fz40, start=Fz40, min=0, max=m*g);

  Real Gxa1(nominal=1, start=1);
  Real Gxa2(nominal=1, start=1);
  Real Gxa3(nominal=1, start=1);
  Real Gxa4(nominal=1, start=1);

  Real Gyk1(nominal=1, start=1);
  Real Gyk2(nominal=1, start=1);
  Real Gyk3(nominal=1, start=1);
  Real Gyk4(nominal=1, start=1);

  Real Bxa1(nominal=Bx1f);
  Real Bxa2(nominal=Bx1f);
  Real Bxa3(nominal=Bx1r);
  Real Bxa4(nominal=Bx1r);

  Real Byk1(nominal=By1f);
  Real Byk2(nominal=By1f);
  Real Byk3(nominal=By1r);
  Real Byk4(nominal=By1r);

  Real Mz_g(start=0,nominal=Fxfmax);
  Real Fx_g(start=0,nominal=Fxfmax);
  Real Fy_g(start=0,nominal=Fyfmax);

  Real Tw1(min=Twfmin, max=Twfmax, nominal=Twrmax);
  Real Tw2(min=Twfmin, max=Twfmax, nominal=Twrmax);
  Real Tw3(min=Twrmin, max=Twrmax, nominal=Twrmax);
  Real Tw4(min=Twrmin, max=Twrmax, nominal=Twrmax);
  
equation

  // motion equations
  Fx_g = (Fx1+Fx2)*cos(delta) - (Fy1+Fy2)*sin(delta)  + Fx3 + Fx4;
  Fy_g = (Fx1+Fx2)*sin(delta) + (Fy1+Fy2)*cos(delta)  + Fy3 + Fy4;
  Mz_g = lf*( (Fx1+Fx2)*sin(delta) + (Fy1+Fy2)*cos(delta) ) - lr*(Fy3+Fy4) + w*( (-Fx1+Fx2)*cos(delta) + (Fy1-Fy2)*sin(delta) - Fx3 + Fx4);

  ( -vy*r + der(vx) + sin(phi)*der(r)*h - sin(theta)*cos(phi)*h*(r^2 + dphi^2 + dtheta^2) + 2*cos(phi)*h*dphi*r + cos(theta)*cos(phi)*h*der(dtheta) - 2*cos(theta)*sin(phi)*dtheta*dphi*h - sin(theta)*sin(phi)*h*der(dphi))*m = Fx_g;
  
  ( der(vy) + vx*r + sin(theta)*cos(phi)*h*der(r) + sin(phi)*h*r^2 + 2*(cos(theta)*cos(phi)*h*dtheta*r - sin(theta)*sin(phi)*h*dphi*r) + sin(phi)*h*dphi^2 - cos(phi)*h*der(dphi))*m = Fy_g;
  
  der(r)*( sin(theta)^2*Ixx + cos(theta)^2*( Izz*cos(phi)^2 + Iyy*sin(phi)^2 ) ) = Mz_g - Fx_g*h*sin(phi) - sin(theta)*cos(phi)*h*Fy_g;
  
  der(dphi)*( Ixx*cos(theta)^2 + sin(theta)^2*sin(phi)^2*Iyy + sin(theta)^2*cos(phi)^2*Izz ) + Kphi*phi + Dphi*dphi = Fy_g*h*cos(phi)*cos(theta) + m*g*h*sin(phi) + r*( r*(Iyy - Izz)*sin(phi)*cos(phi)*cos(theta) + dphi*sin(theta)*sin(phi)*cos(phi)*(Iyy-Izz) + dtheta*(cos(phi)^2*Iyy+sin(phi)^2*Izz) );
  
  der(dtheta)*( Iyy*cos(phi)^2 + Izz*sin(phi)^2 ) + Dth*dtheta + Kth*theta = m*g*h*sin(theta)*cos(phi) - cos(theta)*cos(phi)*h*Fx_g + r*( r*sin(theta)*cos(theta)*( Ixx - Iyy + cos(phi)^2*(Iyy - Izz) ) - dphi*(cos(theta)^2*Ixx + sin(phi)^2*sin(theta)^2*Iyy + sin(theta)^2*cos(phi)^2*Izz) - dtheta*(sin(theta)*sin(phi)*cos(phi)*(Iyy - Izz)) );

  dphi = der(phi);
  der(theta) = dtheta;

  -w*(Fz1-Fz2) = Kphi/2*phi + Dphi/2*dphi;
  -w*(Fz3-Fz4) = Kphi/2*phi + Dphi/2*dphi;
  (Fz1+Fz2)*lf - (Fz3+Fz4)*lr = Kth*theta + Dth*dtheta;
  Fz1 + Fz2 + Fz3 + Fz4 = m*g;

  Tw1 = Twf/2;
  Tw2 = Twf/2;
  Tw3 = Twr/2;
  Tw4 = Twr/2;

  Tw1 - Iw*der(omega1) - Fx1*Rw = 0;
  Tw2 - Iw*der(omega2) - Fx2*Rw = 0;
  Tw3 - Iw*der(omega3) - Fx3*Rw = 0;
  Tw4 - Iw*der(omega4) - Fx4*Rw = 0;

  // slip angles
  vx1 = (vx-r*w)*cos(delta) + (vy+r*lf)*sin(delta);
  vx2 = (vx+r*w)*cos(delta) + (vy+r*lf)*sin(delta);
  vx3 = vx - r*w;
  vx4 = vx + r*w;

  vy1 = (vy+r*lf)*cos(delta) - (vx-r*w)*sin(delta);
  vy2 = (vy+r*lf)*cos(delta) - (vx+r*w)*sin(delta);
  vy3 = vy - r*lr;
  vy4 = vy - r*lr;

  der(alpha1)*sig/vx1 + alpha1 = -atan(vy1/vx1);
  der(alpha2)*sig/vx2 + alpha2 = -atan(vy2/vx2);
  der(alpha3)*sig/vx3 + alpha3 = -atan(vy3/vx3);
  der(alpha4)*sig/vx4 + alpha4 = -atan(vy4/vx4);

  kappa1 = (Rw*omega1 - vx1)/vx1;
  kappa2 = (Rw*omega2 - vx2)/vx2;
  kappa3 = (Rw*omega3 - vx3)/vx3;
  kappa4 = (Rw*omega4 - vx4)/vx4;

  // tire forces
  Bxa1 = Bx1f*cos(atan(Bx2f*kappa1));
  Bxa2 = Bx1f*cos(atan(Bx2f*kappa2));
  Bxa3 = Bx1r*cos(atan(Bx2r*kappa3));
  Bxa4 = Bx1r*cos(atan(Bx2r*kappa4));

  Gxa1 = cos(Cxaf*atan(Bxa1*alpha1));
  Gxa2 = cos(Cxaf*atan(Bxa2*alpha2));
  Gxa3 = cos(Cxar*atan(Bxa3*alpha3));
  Gxa4 = cos(Cxar*atan(Bxa4*alpha4));

  Byk1 = By1f*cos(atan(By2f*(alpha1)));
  Byk2 = By1f*cos(atan(By2f*(alpha2)));
  Byk3 = By1r*cos(atan(By2r*(alpha3)));
  Byk4 = By1r*cos(atan(By2r*(alpha4)));

  Gyk1 = cos(Cykf*atan(Byk1*kappa1));
  Gyk2 = cos(Cykf*atan(Byk2*kappa2));
  Gyk3 = cos(Cykr*atan(Byk3*kappa3));
  Gyk4 = cos(Cykr*atan(Byk4*kappa4));

  Fx10 = muxf*Fz1*sin(Cxf*atan(Bxf*kappa1 - Exf*(Bxf*kappa1 - atan(Bxf*kappa1))));
  Fx20 = muxf*Fz2*sin(Cxf*atan(Bxf*kappa2 - Exf*(Bxf*kappa2 - atan(Bxf*kappa2))));
  Fx30 = muxr*Fz3*sin(Cxr*atan(Bxr*kappa3 - Exr*(Bxr*kappa3 - atan(Bxr*kappa3))));
  Fx40 = muxr*Fz4*sin(Cxr*atan(Bxr*kappa4 - Exr*(Bxr*kappa4 - atan(Bxr*kappa4))));
  
  Fy10 = muyf*Fz1*sin(Cyf*atan(Byf*alpha1 - Eyf*(Byf*alpha1 - atan(Byf*alpha1))));
  Fy20 = muyf*Fz2*sin(Cyf*atan(Byf*alpha2 - Eyf*(Byf*alpha2 - atan(Byf*alpha2))));
  Fy30 = muyr*Fz3*sin(Cyr*atan(Byr*alpha3 - Eyr*(Byr*alpha3 - atan(Byr*alpha3))));
  Fy40 = muyr*Fz4*sin(Cyr*atan(Byr*alpha4 - Eyr*(Byr*alpha4 - atan(Byr*alpha4))));

  Fx1 = Gxa1*Fx10;
  Fx2 = Gxa2*Fx20;
  Fx3 = Gxa3*Fx30;
  Fx4 = Gxa4*Fx40;

  Fy1 = Gyk1*Fy10;
  Fy2 = Gyk2*Fy20;
  Fy3 = Gyk3*Fy30;
  Fy4 = Gyk4*Fy40;

  // global coordinates
  der(psi) = r;
  der(X) = vx*cos(psi) - vy*sin(psi);
  der(Y) = vx*sin(psi) + vy*cos(psi);

end DTWF;
